(* $Id: godi_wget_handler.ml 904 2009-09-16 13:23:06Z gerd $ *)

open Godi_types

let ws_re = Str.regexp "[ \t\r\n]+"

class wget_handler url : file_handler =
  (* We can assume that [url] is always absolute! *)
object
  method url = url

  method list con () =
    failwith "Cannot list remote files (restriction of boot_console)"

  method get con name =
    assert(not (String.contains name '/'));
    let tdir = Godi_file.allocate_tmpdir() in
    let tfile = Godi_file.addname (Godi_file.tmpdir_path tdir) name in
    let eff_url =
      if url.[ String.length url - 1 ] = '/' then
	url ^ name
      else
	url ^ "/" ^ name in
    try
      let wget_options_s =
	Godi_query.wget_options() in
      let wget_options =
	Str.split ws_re wget_options_s in

      Godi_exec.call_cmd
	~stdout:con.protocol
	~stderr:`Like_stdout
	~interactivity:con.interactivity
	( Godi_query.wget(),
	  (Array.of_list
	     (wget_options @
		[ "-O";
		  (Godi_file.slashname tfile);
		  eff_url
		]
	     )
	  )
	);
      (tfile, Some (fun () -> Godi_file.release_tmpdir tdir))
    with
      | Godi_exec.Command_not_found s ->
	  raise(File_handler_error("Command not found: " ^ s))
      | Godi_exec.Command_failure (s,_,_) ->
	  raise(File_handler_error("Command not successful: " ^ s))
end


let init() =
  Godi_file_handler.register "http" (new wget_handler);
  Godi_file_handler.register "ftp" (new wget_handler)

